@@ -27,6 +27,7 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "artwork", kli18nc("OSM::amenity/shop", "Artwork") },
     { "atm", kli18nc("OSM::amenity/shop", "ATM") },
     { "attraction", kli18nc("OSM::amenity/shop", "Attraction") },
+    { "auditorium", kli18nc("OSM::room", "Auditorium") },
     { "baby_goods", kli18nc("OSM::shop", "Baby Goods") },
     { "bag", kli18nc("OSM::amenity/shop", "Bag") },
     { "baggage_claim", kli18nc("OSM::amenity", "Baggage Claim") },
@@ -68,7 +69,9 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "coffee", kli18nc("OSM::amenity/shop", "Coffee") },
     { "community_centre", kli18nc("OSM::amenity", "Community Center") },
     { "computer", kli18nc("OSM::amenity/shop", "Computer") },
+    { "computer_lab", kli18nc("OSM::room", "Computer Lab") },
     { "confectionery", kli18nc("OSM::amenity/shop", "Confectionery") },
+    { "conference", kli18nc("OSM::room", "Conference Room") },
     { "conference_centre", kli18nc("OSM::amenity", "Conference Center") },
     { "consulate", kli18nc("OSM::diplomatic", "Consulate") },
     { "convenience", kli18nc("OSM::amenity/shop", "Convenience Store") },
@@ -86,6 +89,7 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "doctors", kli18nc("OSM::healthcare", "Doctor") },
     { "doityourself", kli18nc("OSM::amenity/shop", "Hardware Store") },
     { "drinking_water", kli18nc("OSM::amenity/shop", "Drinking Water") },
+    { "drinks", kli18nc("OSM::vending", "Drinks") },
     { "dry_cleaning", kli18nc("OSM::amenity/shop", "Dry Cleaning") },
     { "electrical", kli18nc("OSM::shop", "Electrical") },
     { "electronics", kli18nc("OSM::amenity/shop", "Electronics") },
@@ -131,6 +135,7 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "jewelry", kli18nc("OSM::amenity/shop", "Jewelry") },
     { "kiosk", kli18nc("OSM::amenity/shop", "Kiosk") },
     { "kitchen", kli18nc("OSM::room", "Kitchen") },
+    { "laboratory", kli18nc("OSM::room", "Laboratory") },
     { "laundry", kli18nc("OSM::amenity/shop", "Laundry") },
     { "lecture", kli18nc("OSM::room", "Lecture Hall") },
     { "lecture_hall", kli18nc("OSM::room", "Lecture Hall") },
@@ -145,6 +150,7 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "mall", kli18nc("OSM::amenity/shop", "Mall") },
     { "marketplace", kli18nc("OSM::amenity", "Marketplace") },
     { "medical_supply", kli18nc("OSM::amenity/shop", "Medical Supply") },
+    { "meeting", kli18nc("OSM::room", "Meeting Room") },
     { "memorial", kli18nc("OSM::historic", "Memorial") },
     { "mobile_phone", kli18nc("OSM::amenity/shop", "Mobile Phone") },
     { "money_transfer", kli18nc("OSM::amenity/shop", "Money Transfer") },
@@ -182,6 +188,7 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "pub", kli18nc("OSM::amenity/shop", "Pub") },
     { "public_transport_tickets", kli18nc("OSM::amenity/shop", "Public Transport Tickets") },
     { "railway_station", kli18nc("OSM::building", "Train Station") },
+    { "reception", kli18nc("place to welcome guests, e.g. in a hotel", "Reception") },
     { "recycling", kli18nc("OSM::amenity/shop", "Recycling") },
     { "research_institute", kli18nc("OSM::amenity", "Research Institute") },
     { "restaurant", kli18nc("OSM::amenity/shop", "Restaurant") },
@@ -190,6 +197,7 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "scooter_rental", kli18nc("OSM::amenity/shop", "Kick Scooter Rental") },
     { "seafood", kli18nc("OSM::amenity/shop", "Seafood") },
     { "second_hand", kli18nc("OSM::shop", "Second Hand") },
+    { "seminar_room", kli18nc("OSM::room", "Seminar Room") },
     { "shoe_repair", kli18nc("OSM::shop", "Shoe Repair") },
     { "shoes", kli18nc("OSM::shop", "Shoes") },
     { "shop", kli18nc("OSM::amenity/shop", "Shop") },
@@ -201,15 +209,18 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "sports_hall", kli18nc("OSM::amenity", "Sports Hall") },
     { "stationery", kli18nc("OSM::amenity/shop", "Stationery") },
     { "supermarket", kli18nc("OSM::amenity/shop", "Supermarket") },
+    { "sweets", kli18nc("OSM::vending", "Sweets") },
     { "tailor", kli18nc("OSM::amenity/shop", "Tailor") },
     { "tatoo", kli18nc("OSM::amenity/shop", "Tattoo") },
     { "taxi", kli18nc("OSM::amenity/shop", "Taxi") },
     { "tea", kli18nc("OSM::amenity/shop", "Tea") },
     { "telecommunication", kli18nc("OSM::shop", "Telecommunication") },
+    { "terrace", kli18nc("OSM::room", "Terrace") },
     { "theatre", kli18nc("OSM::amenity/shop", "Theatre") },
     { "theme_park", kli18nc("OSM::tourism", "Theme Park") },
     { "ticket", kli18nc("OSM::amenity/shop", "Tickets") },
     { "tobacco", kli18nc("OSM::amenity/shop", "Tobacco") },
+    { "toilet", kli18nc("OSM::amenity/shop", "Toilets") },
     { "toilets", kli18nc("OSM::amenity/shop", "Toilets") },
     { "townhall", kli18nc("OSM::amenity/shop", "Town Hall") },
     { "toys", kli18nc("OSM::amenity/shop", "Toys") },
@@ -218,6 +229,7 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "travel_agent", kli18nc("OSM::amenity/shop", "Travel Agency") },
     { "university", kli18nc("OSM::amenity/shop", "University") },
     { "variety_store", kli18nc("OSM::amenity/shop", "Variety Store") },
+    { "vending_machine", kli18nc("OSM::amenity", "Vending Machine") },
     { "veterinary", kli18nc("OSM::amenity", "Veterinary") },
     { "video_games", kli18nc("OSM::amenity/shop", "Video Games") },
     { "viewpoint", kli18nc("OSM::tourism", "Viewpoint") },
@@ -226,6 +238,7 @@ static constexpr const ValueMapEntry amenity_map[] = {
     { "waiting_room", kli18nc("OSM::amenity/shop", "Waiting Area") },
     { "waste_basket", kli18nc("OSM::amenity", "Waste Basket") },
     { "watches", kli18nc("OSM::shop", "Watches") },
+    { "water", kli18nc("OSM::vending", "Water") },
     { "wine", kli18nc("OSM::amenity/shop", "Wine") },
     { "zoo", kli18nc("OSM::tourism", "Zoo") },
 };
@@ -345,4 +358,12 @@ static constexpr const ValueMapEntry cuisine_map[] = {
 };
 static_assert(isSortedLookupTable(cuisine_map), "cuising map is not sorted!");
 
+// gender neutral/gender segregated facilities
+static constexpr const ValueMapEntry gender_type_map[] = {
+    { "female", kli18nc("OSM::gender_segregation", "female") },
+    { "male", kli18nc("OSM::gender_segregation", "male") },
+    { "unisex", kli18nc("OSM::gender_segregation", "unisex") },
+};
+static_assert(isSortedLookupTable(gender_type_map), "gender type map is not sorted!");
+
 }
