@@ -6,6 +6,7 @@
 
 #include "osmelement.h"
 
+#include <QJSValueIterator>
 #include <QLocale>
 
 using namespace KOSMIndoorMap;
@@ -23,6 +24,11 @@ bool OSMElement::isNull() const
     return m_element.type() == OSM::Type::Null;
 }
 
+qint64 OSMElement::id() const
+{
+    return m_element.id();
+}
+
 QString OSMElement::name() const
 {
     return QString::fromUtf8(m_element.tagValue(OSM::Languages::fromQLocale(QLocale()), "name"));
@@ -33,9 +39,28 @@ QString OSMElement::url() const
     return m_element.url();
 }
 
-QString OSMElement::tagValue(const QString &key) const
+QPointF OSMElement::center() const
 {
-    return QString::fromUtf8(m_element.tagValue(key.toUtf8().constData()));
+    const auto c = m_element.center();
+    return {c.lonF(), c.latF() };
+}
+
+QString OSMElement::tagValue(const QJSValue &key) const
+{
+    if (key.isString()) {
+        return QString::fromUtf8(m_element.tagValue(key.toString().toUtf8().constData()));
+    }
+    if (key.isArray()) {
+        for (QJSValueIterator it(key); it.hasNext();) {
+            it.next();
+            auto v = m_element.tagValue(it.value().toString().toUtf8().constData());
+            if (!v.isEmpty()) {
+                return QString::fromUtf8(v);
+            }
+        }
+    }
+
+    return {};
 }
 
 OSM::Element OSMElement::element() const
