@@ -9,6 +9,8 @@
 
 #include "kosmindoormap_export.h"
 
+#include <KOSMIndoorMap/NetworkAccessManagerFactory>
+
 #include <QDateTime>
 #include <QFile>
 #include <QObject>
@@ -56,7 +58,7 @@ class TileCache : public QObject
 {
     Q_OBJECT
 public:
-    explicit TileCache(QObject *parent = nullptr);
+    explicit TileCache(const NetworkAccessManagerFactory &namFactory,  QObject *parent = nullptr);
     ~TileCache();
 
     /** Returns the path to the cached content of @p tile, if present locally. */
@@ -88,7 +90,7 @@ private:
     void downloadFinished(QNetworkReply *reply, Tile tile);
     void updateTtl(const QString &filePath, const QDateTime &ttl);
 
-    QNetworkAccessManager *m_nam;
+    NetworkAccessManagerFactory m_nam;
     QFile m_output;
     std::deque<Tile> m_pendingDownloads;
 };
