@@ -15,7 +15,9 @@
 #include "openinghourscache_p.h"
 #include "texturecache_p.h"
 #include "../style/mapcssdeclaration_p.h"
+#include "../style/mapcssexpressioncontext_p.h"
 #include "../style/mapcssstate_p.h"
+#include "../style/mapcssvalue_p.h"
 
 #include <KOSMIndoorMap/MapData>
 #include <KOSMIndoorMap/MapCSSResult>
@@ -31,6 +33,9 @@
 #include <QElapsedTimer>
 #include <QGuiApplication>
 #include <QPalette>
+#include <QScopedValueRollback>
+
+using namespace Qt::Literals::StringLiterals;
 
 namespace KOSMIndoorMap {
 class SceneControllerPrivate
@@ -129,6 +134,7 @@ void SceneController::updateScene(SceneGraph &sg) const
     d->m_dirty = false;
 
     sg.beginSwap();
+    std::for_each(d->m_overlaySources.begin(), d->m_overlaySources.end(), std::mem_fn(&AbstractOverlaySource::beginSwap));
     updateCanvas(sg);
 
     if (d->m_data.isEmpty()) { // if we don't have map data yet, we just need to get canvas styling here
@@ -177,16 +183,19 @@ void SceneController::updateScene(SceneGraph &sg) const
     // update overlay elements
     d->m_overlay = true;
     for (const auto &overlaySource : d->m_overlaySources) {
+        QScopedValueRollback tranientNodes(d->m_data.dataSet().transientNodes, overlaySource->transientNodes());
         overlaySource->forEach(d->m_view->level(), [this, &geoBbox, &sg](OSM::Element e, int floorLevel) {
             if (OSM::intersects(geoBbox, e.boundingBox()) && e.type() != OSM::Type::Null) {
                 updateElement(e, floorLevel, sg);
             }
         });
+        d->m_data.dataSet().transientNodes = nullptr;
     }
     d->m_overlay = false;
 
     sg.zSort();
     sg.endSwap();
+    std::for_each(d->m_overlaySources.begin(), d->m_overlaySources.end(), std::mem_fn(&AbstractOverlaySource::endSwap));
 
     qCDebug(RenderLog) << "updated scenegraph took" << sgUpdateTimer.elapsed() << "ms";
 }
@@ -223,9 +232,10 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg) c
     state.floorLevel = d->m_view->level();
     state.openingHours = &d->m_openingHours;
     state.state = d->m_hoverElement == e ? MapCSSElementState::Hovered : MapCSSElementState::NoState;
-    d->m_styleSheet->evaluate(std::move(state), d->m_styleResult);
+    d->m_styleSheet->initializeState(state);
+    d->m_styleSheet->evaluate(state, d->m_styleResult);
     for (const auto &result : d->m_styleResult.results()) {
-        updateElement(e, level, sg, result);
+        updateElement(state, level, sg, result);
     }
 }
 
@@ -234,25 +244,25 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg) c
     return std::any_of(s.begin(), s.end(), [](QChar c) { return !c.isLetter(); });
 }
 
-void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, const MapCSSResultLayer &result) const
+void SceneController::updateElement(const MapCSSState &state, int level, SceneGraph &sg, const MapCSSResultLayer &result) const
 {
     if (result.hasAreaProperties()) {
         PolygonBaseItem *item = nullptr;
         std::unique_ptr<SceneGraphItemPayload> baseItem;
-        if (e.type() == OSM::Type::Relation && e.tagValue(d->m_typeTag) == "multipolygon") {
-            baseItem = sg.findOrCreatePayload<MultiPolygonItem>(e, level, result.layerSelector());
+        if (state.element.type() == OSM::Type::Relation && state.element.tagValue(d->m_typeTag) == "multipolygon") {
+            baseItem = sg.findOrCreatePayload<MultiPolygonItem>(state.element, level, result.layerSelector());
             auto i = static_cast<MultiPolygonItem*>(baseItem.get());
             if (i->path.isEmpty()) {
-                i->path = createPath(e, d->m_labelPlacementPath);
+                i->path = createPath(state.element, d->m_labelPlacementPath);
             } else if (result.hasLabelProperties()) {
                 SceneGeometry::outerPolygonFromPath(i->path, d->m_labelPlacementPath);
             }
             item = i;
         } else {
-            baseItem = sg.findOrCreatePayload<PolygonItem>(e, level, result.layerSelector());
+            baseItem = sg.findOrCreatePayload<PolygonItem>(state.element, level, result.layerSelector());
             auto i = static_cast<PolygonItem*>(baseItem.get());
             if (i->polygon.isEmpty()) {
-                i->polygon = createPolygon(e);
+                i->polygon = createPolygon(state.element);
             }
             d->m_labelPlacementPath = i->polygon;
             item = i;
@@ -267,8 +277,8 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
         initializePen(item->casingPen);
         for (auto decl : result.declarations()) {
             applyGenericStyle(decl, item);
-            applyPenStyle(e, decl, item->pen, lineOpacity, item->penWidthUnit);
-            applyCasingPenStyle(e, decl, item->casingPen, casingOpacity, item->casingPenWidthUnit);
+            applyPenStyle(state.element, decl, item->pen, lineOpacity, item->penWidthUnit);
+            applyCasingPenStyle(state.element, decl, item->casingPen, casingOpacity, item->casingPenWidthUnit);
             switch (decl->property()) {
                 case MapCSSProperty::FillColor:
                     item->fillBrush.setColor(decl->colorValue());
@@ -303,12 +313,12 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
             item->textureBrush.setStyle(Qt::NoBrush);
         }
 
-        addItem(sg, e, level, result, std::move(baseItem));
+        addItem(sg, state, level, result, std::move(baseItem));
     } else if (result.hasLineProperties()) {
-        auto baseItem = sg.findOrCreatePayload<PolylineItem>(e, level, result.layerSelector());
+        auto baseItem = sg.findOrCreatePayload<PolylineItem>(state.element, level, result.layerSelector());
         auto item = static_cast<PolylineItem*>(baseItem.get());
         if (item->path.isEmpty()) {
-            item->path = createPolygon(e);
+            item->path = createPolygon(state.element);
         }
 
         double lineOpacity = 1.0;
@@ -318,14 +328,14 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
         initializePen(item->casingPen);
         for (auto decl : result.declarations()) {
             applyGenericStyle(decl, item);
-            applyPenStyle(e, decl, item->pen, lineOpacity, item->penWidthUnit);
-            applyCasingPenStyle(e, decl, item->casingPen, casingOpacity, item->casingPenWidthUnit);
+            applyPenStyle(state.element, decl, item->pen, lineOpacity, item->penWidthUnit);
+            applyCasingPenStyle(state.element, decl, item->casingPen, casingOpacity, item->casingPenWidthUnit);
         }
         finalizePen(item->pen, lineOpacity);
         finalizePen(item->casingPen, casingOpacity);
 
         d->m_labelPlacementPath = item->path;
-        addItem(sg, e, level, result, std::move(baseItem));
+        addItem(sg, state, level, result, std::move(baseItem));
     }
 
     if (result.hasLabelProperties()) {
@@ -336,8 +346,10 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
         }
 
         if (textDecl) {
-            if (!textDecl->keyValue().isEmpty()) {
-                text = QString::fromUtf8(e.tagValue(d->m_langs, textDecl->keyValue().constData()));
+            if (textDecl->hasExpression()) {
+                text = QString::fromUtf8(textDecl->evaluateExpression({state, result}).asString());
+            } else if (!textDecl->keyValue().isEmpty()) {
+                text = QString::fromUtf8(state.element.tagValue(d->m_langs, textDecl->keyValue().constData()));
             } else {
                 text = textDecl->stringValue();
             }
@@ -346,7 +358,7 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
         const auto iconDecl = result.declaration(MapCSSProperty::IconImage);
 
         if (!text.isEmpty() || iconDecl) {
-            auto baseItem = sg.findOrCreatePayload<LabelItem>(e, level, result.layerSelector());
+            auto baseItem = sg.findOrCreatePayload<LabelItem>(state.element, level, result.layerSelector());
             auto item = static_cast<LabelItem*>(baseItem.get());
             item->text.setText(text);
             item->textIsSet = !text.isEmpty();
@@ -417,16 +429,16 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
                         break;
                     case MapCSSProperty::IconImage:
                         if (!decl->keyValue().isEmpty()) {
-                            iconData.name = QString::fromUtf8(e.tagValue(decl->keyValue().constData()));
+                            iconData.name = QString::fromUtf8(state.element.tagValue(decl->keyValue().constData()));
                         } else {
                             iconData.name = decl->stringValue();
                         }
                         break;
                     case MapCSSProperty::IconHeight:
-                        item->iconSize.setHeight(PenWidthUtil::penWidth(e, decl, item->iconHeightUnit));
+                        item->iconSize.setHeight(PenWidthUtil::penWidth(state.element, decl, item->iconHeightUnit));
                         break;
                     case MapCSSProperty::IconWidth:
-                        item->iconSize.setWidth(PenWidthUtil::penWidth(e, decl, item->iconWidthUnit));
+                        item->iconSize.setWidth(PenWidthUtil::penWidth(state.element, decl, item->iconWidthUnit));
                         break;
                     case MapCSSProperty::IconColor:
                     {
@@ -467,7 +479,7 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
                     item->pos = SceneGeometry::polylineMidPoint(d->m_labelPlacementPath);
                 }
                 if (item->pos.isNull()) {
-                    item->pos = d->m_view->mapGeoToScene(e.center()); // node or something failed above
+                    item->pos = d->m_view->mapGeoToScene(state.element.center()); // node or something failed above
                 }
             }
 
@@ -505,6 +517,11 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
                 opt.setAlignment(Qt::AlignHCenter);
                 opt.setWrapMode(item->text.textWidth() > 0.0 ? QTextOption::WordWrap : QTextOption::NoWrap);
                 item->text.setTextOption(opt);
+
+                if (item->text.text().contains('\n'_L1) || item->text.textWidth() > 0) {
+                    item->isComplexText = true;
+                }
+
                 // do not use QStaticText::prepare here:
                 // the vast majority of text items will likely not be shown at all for being overlapped or out of view
                 // and pre-computing them is too expensive. Instead this will happen as needed on first use, for only
@@ -531,7 +548,7 @@ void SceneController::updateElement(OSM::Element e, int level, SceneGraph &sg, c
             }
 
             if (!item->icon.isNull() || !item->text.text().isEmpty()) {
-                addItem(sg, e, level, result, std::move(baseItem));
+                addItem(sg, state, level, result, std::move(baseItem));
             }
         }
     }
@@ -716,23 +733,20 @@ void SceneController::finalizePen(QPen &pen, double opacity) const
     }
 }
 
-void SceneController::addItem(SceneGraph &sg, OSM::Element e, int level, const MapCSSResultLayer &result, std::unique_ptr<SceneGraphItemPayload> &&payload) const
+void SceneController::addItem(SceneGraph &sg, const MapCSSState &state, int level, const MapCSSResultLayer &result, std::unique_ptr<SceneGraphItemPayload> &&payload) const
 {
     SceneGraphItem item;
-    item.element = e;
+    item.element = state.element;
     item.layerSelector = result.layerSelector();
     item.level = level;
     item.payload = std::move(payload);
 
     // get the OSM layer, if set
     if (!d->m_overlay) {
-        auto layerStr = result.tagValue(d->m_layerTag);
-        if (layerStr.isNull()) {
-            layerStr = e.tagValue(d->m_layerTag);
-        }
-        if (!layerStr.isEmpty()) {
+        const auto layerStr = result.resolvedTagValue(d->m_layerTag, state);
+        if (layerStr && !(*layerStr).isEmpty()) {
             bool success = false;
-            const auto layer = layerStr.toInt(&success);
+            const auto layer = (*layerStr).toInt(&success);
             if (success) {
 
                 // ### Ignore layer information when it matches the level
@@ -748,7 +762,7 @@ void SceneController::addItem(SceneGraph &sg, OSM::Element e, int level, const M
                     item.layer = layer;
                 }
             } else {
-                qCWarning(Log) << "Invalid layer:" << e.url() << layerStr;
+                qCWarning(Log) << "Invalid layer:" << state.element.url() << *layerStr;
             }
         }
     } else {
